package eu.mhsl.minecraft.pixelpic.render.registry;

import eu.mhsl.minecraft.pixelpic.render.model.AbstractModel;
import eu.mhsl.minecraft.pixelpic.render.model.Model;
import org.bukkit.Color;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.block.data.BlockData;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.*;

import static eu.mhsl.minecraft.pixelpic.render.registry.DefaultModelRegistry.TEXTURE_SIZE;

public class AdvancedModelRegistry implements ModelRegistry {
    private final Map<Material, Map<BlockData, Model>> modelMap = new HashMap<>();

    @Override
    public void initialize() {
//        registerModel(Material.STONE, AbstractModel.Builder.createSimple(getTextureArray(Material.STONE.getBlockTranslationKey().replace("block.minecraft.", ""))).build());
        for (Material currentMaterial : Material.values()) {
            if(!currentMaterial.isBlock()) continue;
            List<String> blockTextures = getBlockTextures(currentMaterial);
            for (String blockTexture : blockTextures) {
                try {
                    registerModel(currentMaterial, AbstractModel.Builder.createSimple(getTextureArray(blockTexture)).build());
                } catch (Exception ignored) { }
            }
        }
    }

    @Override
    public Model getModel(Block block) {
        return ModelRegistry.super.getModel(block);
    }

    @Override
    public Model getModel(Material material, BlockData blockData) {
        return modelMap.computeIfAbsent(material, key -> new HashMap<>()).getOrDefault(blockData,
            blockData == null ? getDefaultModel()
                : modelMap.get(material).getOrDefault(null, getDefaultModel()));
    }

    @Override
    public Model getDefaultModel() {
        return AbstractModel.Builder.createStatic(Color.PURPLE.asRGB()).build();
    }

    private void registerModel(Material material, Model blockModel) {
        modelMap.computeIfAbsent(material, key -> new HashMap<>())
            .put(null, blockModel);
    }

    private List<String> getBlockTextures(Material material) {
        if(!material.isBlock()) return new ArrayList<>();
        String blockName = material.name().toLowerCase();
        try {
            if(!Arrays.deepEquals(getTextureArray(blockName), new int[0][0])) {
                return List.of(blockName);
            }
        } catch (Exception e) {
            String[] possibleVariants = {
                blockName + "_top",
                blockName + "_side",
                blockName + "_bottom",
                blockName + "_front",
                blockName + "_back",
                blockName + "_left",
                blockName + "_right"
            };
            List<String> results = new ArrayList<>();
            for (String variant : possibleVariants) {
                try {
                    if(!Arrays.deepEquals(getTextureArray(variant), new int[0][0])) {
                        results.add(variant);
                    }
                } catch (Exception ignored) { }
            }
            return results;
        }
        return new ArrayList<>();
    }

    private int[][] getTextureArray(String textureName) {
        int[][] texture = new int[TEXTURE_SIZE][TEXTURE_SIZE];
        BufferedImage img = null;
        URL url = this.getClass().getClassLoader().getResource(String.format("textures/block/%s.png", textureName));
        if (url == null) {
            throw new RuntimeException("Block Texture Resource not found.");
        }
        try (InputStream input = url.openConnection().getInputStream()) {
            img = ImageIO.read(input);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        for (int pixelY = 0; pixelY < TEXTURE_SIZE; pixelY++) {
            for (int pixelX = 0; pixelX < TEXTURE_SIZE; pixelX++) {
                texture[pixelY][pixelX] = img.getRGB(pixelX, pixelY);
            }
        }

        return texture;
    }
}
